
import os
import sys
import json
from pathlib import Path
from unittest.mock import Mock, patch, MagicMock
from langchain_core.messages import AIMessage

from alayaflow.component.chat_model import mk_chat_model_deepseek

sys.path.insert(0, str(Path(__file__).parent.parent.parent.parent / "src"))
import pytest

# import function to be tested
from alayaflow.component.intent_classifier import execute_intent_classification, IntentionResult

class TestIntentClassifier:
    """测试execute_intent_classification函数的测试类"""

    @pytest.fixture
    def mock_model(self):
        """创建一个模拟的ChatOpenAI模型"""
        mock_model = Mock()
        mock_model.invoke = Mock()
        return mock_model

    @pytest.fixture
    def test_data(self):
        """测试数据集"""
        return {
            "scenario_1": {
                "query": "今天天气怎么样？",
                "intents": ["查询天气", "播放音乐", "设置提醒"],
                "expected_response": {"classification_id": 0, "reason": "用户明确询问了天气情况"},
            },
            "scenario_2": {
                "query": "我想听周杰伦的歌",
                "intents": ["查询天气", "播放音乐", "设置提醒"],
                "expected_response": {"classification_id": 1, "reason": "用户想要听音乐"},
            },
            "scenario_3": {
                "query": "帮我设置明天上午9点的闹钟",
                "intents": ["查询天气", "播放音乐", "设置提醒"],
                "expected_response": {"classification_id": 2, "reason": "用户需要设置提醒"},
            },
            "scenario_4": {
                "query": "这个问题不匹配任何意图",
                "intents": ["查询天气", "播放音乐", "设置提醒"],
                "expected_response": {"classification_id": -1, "reason": "与所有选项均不匹配"},
            },
            "scenario_5": {
                "query": "测试空意图列表",
                "intents": [],
                "expected_response": {"classification_id": -1, "reason": "input intent list is empty"},
            }
        }

    def test_intent_classifier_with_empty_intents(self, mock_model, test_data):
        """测试空意图列表的情况"""
        scenario = test_data["scenario_5"]
        query = scenario["query"]
        intents = scenario["intents"]
        expected_response = scenario["expected_response"]

        # 设置mock
        mock_model.invoke = Mock(return_value=expected_response)

        # 调用被测试函数
        result = execute_intent_classification(query, intents, model=mock_model)

        # 验证结果
        assert result == IntentionResult(**expected_response)
        # 验证模型方法被调用
        mock_model.invoke.assert_not_called()

    def test_intent_classification_successful_match(self, mock_model, test_data):
        """测试成功匹配意图的情况"""
        scenario = test_data["scenario_1"]
        query = scenario["query"]
        intents = scenario["intents"]
        expected_response = scenario["expected_response"]

        # 创建模拟的invoke方法
        def mock_invoke(messages: list, response_format: dict) -> AIMessage:
            """模拟的invoke方法，检查prompt内容并返回预设响应"""

            # 检查query是否在content中
            assert len(messages) == 2, "messages should have 2 elements"
            sys_role, sys_content = messages[0]
            user_role, user_content = messages[1]
            assert sys_role == "system", "First message role must be 'system'"
            assert user_role == "user", "Second message role must be 'user'"

            # 检查所有intent是否在content中
            for intent in intents:
                assert intent in sys_content, f"Intent '{intent}' not found in user prompt content"

            # 检查query是否在content中
            assert query in user_content, f"Query '{query}' not found in user prompt content"

            # 检查response_format是否为json_object
            assert response_format == {"type": "json_object"}, "response_format should be {'type': 'json_object'}"

            # 返回预设的响应
            return AIMessage(content=json.dumps(expected_response))

        # 设置mock
        mock_model.invoke = Mock(side_effect=mock_invoke)

        # 调用被测试函数
        result = execute_intent_classification(query, intents, model=mock_model)

        # 验证结果
        assert result == IntentionResult(**expected_response)
        # 验证模型方法被调用
        mock_model.invoke.assert_called_once()

    def test_intent_classification_fallback(self, mock_model, test_data):
        """测试回退到fallback_index的情况"""
        scenario = test_data["scenario_4"]
        query = scenario["query"]
        intents = scenario["intents"]
        expected_response = scenario["expected_response"]

        # 创建模拟的invoke方法
        def mock_invoke(messages: list, response_format: dict) -> AIMessage:
            """模拟的invoke方法，检查prompt内容并返回预设响应"""

            # 检查query是否在content中
            assert len(messages) == 2, "messages should have 2 elements"
            sys_role, sys_content = messages[0]
            user_role, user_content = messages[1]
            assert sys_role == "system", "First message role must be 'system'"
            assert user_role == "user", "Second message role must be 'user'"

            # 检查所有intent是否在content中
            for intent in intents:
                assert intent in sys_content, f"Intent '{intent}' not found in user prompt content"

            # 检查query是否在content中
            assert query in user_content, f"Query '{query}' not found in user prompt content"

            # 检查response_format是否为json_object
            assert response_format == {"type": "json_object"}, "response_format should be {'type': 'json_object'}"

            # 返回预设的响应
            return AIMessage(content=json.dumps(expected_response))

        # 设置mock
        mock_model.invoke = Mock(side_effect=mock_invoke)

        # 调用被测试函数
        fallback_index = -1
        result = execute_intent_classification(query, intents, model=mock_model, fallback_index=fallback_index)

        # 验证结果
        assert result == IntentionResult(**expected_response)
        assert result.classification_id == fallback_index
        
        # 验证模型方法被调用
        mock_model.invoke.assert_called_once()


    def test_intent_classifier_multiple_scenarios(self, mock_model, test_data):
        """测试多个场景"""
        scenarios_to_test = ["scenario_1", "scenario_2", "scenario_3"]

        for scenario_name in scenarios_to_test:
            scenario = test_data[scenario_name]
            query = scenario["query"]
            intents = scenario["intents"]
            expected_response = scenario["expected_response"]

            # 重置mock
            mock_model.reset_mock()

            # 设置模拟响应
            mock_model.invoke.return_value = AIMessage(content=json.dumps(expected_response))

            # 调用被测试函数
            result = execute_intent_classification(query, intents, model=mock_model)

            # 验证结果
            assert result == IntentionResult(**expected_response)

            # 验证模型被调用
            mock_model.invoke.assert_called_once()

            # 验证调用参数
            call_args = mock_model.invoke.call_args
            messages = call_args[0][0]  # 位置参数的第1个值 → messages列表
            response_format = call_args[1]["response_format"] # 关键字参数的response_format值
            # 检查query是否在content中
            assert len(messages) == 2, "messages should have 2 elements"
            sys_role, sys_content = messages[0]
            user_role, user_content = messages[1]
            assert sys_role == "system", "First message role must be 'system'"
            assert user_role == "user", "Second message role must be 'user'"

            # 检查所有intent是否在content中
            for intent in intents:
                assert intent in sys_content, f"Intent '{intent}' not found in user prompt content"

            # 检查query是否在content中
            assert query in user_content, f"Query '{query}' not found in user prompt content"

            # 检查response_format是否为json_object
            assert response_format == {"type": "json_object"}, "response_format should be {'type': 'json_object'}"
    
    @pytest.mark.skipif(
        not os.getenv("ENABLE_REAL_LLM_TEST"),
        reason="未开启真实LLM测试开关（设置ENABLE_REAL_LLM_TEST=true可运行）"
    )
    def test_intent_classifier_multiple_scenarios_real(self, test_data):
        """测试多个场景"""
        scenarios_to_test = ["scenario_1", "scenario_2", "scenario_3"]

        for scenario_name in scenarios_to_test:
            scenario = test_data[scenario_name]
            query = scenario["query"]
            intents = scenario["intents"]
            expected_response = scenario["expected_response"]

            model = mk_chat_model_deepseek()

            # 调用被测试函数
            result = execute_intent_classification(query, intents, model=model)

            # 验证结果
            assert isinstance(result, IntentionResult), "result should be an instance of IntentionResult"
            assert result.classification_id == expected_response["classification_id"]
    

if __name__ == "__main__":
    pytest.main([__file__, "-v"])




from pathlib import Path
import json

from alayaflow.workflow import WorkflowInfo
from alayaflow.utils.logger import AlayaFlowLogger

logger = AlayaFlowLogger()

def get_metadata():
    try:
        wf_dir = Path(__file__).parent
        meta_path = wf_dir / "metadata.json"
        meta = json.loads(meta_path.read_text())
        meta.update({"wf_dir": wf_dir})
        return WorkflowInfo(**meta)
    except Exception as e:
        logger.error(f"Error loading metadata: {e}")
        raise RuntimeError(f"Error loading metadata: {e}")



from typing import Generator, Dict, AsyncGenerator

from alayaflow.execution.executors.base_executor import BaseExecutor
from alayaflow.workflow.workflow_manager import WorkflowManager

class WorkerExecutor(BaseExecutor):
    def __init__(self, workflow_manager: WorkflowManager):
        self.workflow_manager = workflow_manager
    
    def execute_stream(self, workflow_id: str, version: str, inputs: dict, context: dict, metadata: dict = None) -> Generator[Dict, None, None]:
        raise NotImplementedError("worker executor not supported yet")

    def execute_stream_async(self, workflow_id: str, version: str, inputs: dict, context: dict, metadata: dict = None) -> AsyncGenerator[Dict, None]:
        raise NotImplementedError("worker executor not supported yet")


import asyncio
import json
import httpx
import os

BASE_URL = "http://100.64.0.34:12000"       # dbg21

async def test_health():
    async with httpx.AsyncClient(base_url=BASE_URL) as client:
        response = await client.get("/health")
        if response.status_code != 200:
            print(f"Health Check Status Code: {response.status_code}")
            return
        print(f"Health Check: {response.json()}")

async def test_list_workflows():
    async with httpx.AsyncClient(base_url=BASE_URL, timeout=30.0) as client:
        response = await client.get("/workflows/available")
        if response.status_code != 200:
            print(f"List Available Workflows Status Code: {response.status_code}")
            return
        print(f"Available Workflows: {response.json()}")

async def test_register_models():
    async with httpx.AsyncClient(base_url=BASE_URL) as client:
        response = await client.post("/models/register", json={
            "profiles": [
                {
                    # Local used fields
                    "name": "DeepSeek Chat",
                    "model_id": "deepseek-chat",
                    "provider_name": "DeepSeek",
                    # Connection credentials
                    "model_name": "deepseek-chat",
                    "base_url": "https://api.deepseek.com/v1",
                    "api_key": os.getenv("DEEPSEEK_API_KEY"),
                }
            ]
        })
        if response.status_code != 200:
            print(f"Register Models Status Code: {response.status_code}")
            return
        print(f"Register Models: {response.json()}")

async def test_load_workflow():
    async with httpx.AsyncClient(base_url=BASE_URL, timeout=30.0) as client:
        response = await client.post("/workflows/load", json={
            "workflow_id": "alayamem_chat",
            "version": "1.0.0",
            "init_args": {
                "alayamem_url": "http://100.64.0.5:5556",
            },
        })
        if response.status_code != 200:
            print(f"Load Workflow Status Code: {response.status_code}")
            return
        print(f"Load Workflow: {response.json()}")

async def test_streaming_execution():
    """测试流式执行接口"""
    url = f"{BASE_URL}/workflows/execute/stream"
    
    payload = {
        "workflow_id": "alayamem_chat",
        "version": "1.0.0",
        "inputs": {
            "messages": [{"role": "user", "content": "你好"}]
        },
        "context": {
            "user_id": "user",
            "session_id": "test_session_multi_turn",
            "chat_model_id": "deepseek-chat",
        },
        "metadata": {
            "project": {
                "name": "test",
                "id": "cmkaw73cu000ap607qkp34k1i"
            },
            "org": {
                "name": "sustech",
                "id": "cmkaw5tyw0001p607w96xf0j0"
            }
        }
    }

    print("\n--- Starting Streaming Execution ---")
    async with httpx.AsyncClient(timeout=60.0) as client:
        # 使用 astream 发起流式请求
        async with client.stream("POST", url, json=payload) as response:
            if response.status_code != 200:
                print(f"Error: {response.status_code}, {await response.aread()}")
                return

            # 逐行读取响应 (NDJSON)
            async for line in response.aiter_lines():
                if not line:
                    continue
                
                try:
                    event_data = json.loads(line)
                    # 打印实时收到的事件
                    print(f"[Stream Event]: {event_data}")
                    
                    # 检查是否有错误事件
                    if event_data.get("event") == "error":
                        print(f"Error in stream: {event_data.get('error')}")
                        
                except json.JSONDecodeError:
                    print(f"[Raw Data]: {line}")

    print("--- Streaming Execution Finished ---\n")

async def main():
    # 1. 检查健康状态
    try:
        await test_health()
    except httpx.ConnectError:
        print("无法连接到服务器，请确保 server.py 正在运行 (uvicorn server:app --reload)")
        return

    # 2. 列出工作流 (alayahub 未正确配置的话，可能会比较长)
    await test_list_workflows()

    # 3. 注册模型
    await test_register_models()

    # 4. 加载工作流
    await test_load_workflow()

    # 5. 执行流式调用
    await test_streaming_execution()

if __name__ == "__main__":
    asyncio.run(main())



from langgraph.runtime import Runtime

from alayaflow.component.rerank_node import JinaRerankerComponent
from alayaflow.utils.logger import AlayaFlowLogger

from ..schemas import TaskState, WorkflowContext

logger = AlayaFlowLogger()


def create_rerank_node():
    RERANK_MODEL_ID= "jina-reranker"
    TOP_N = 5
    
    def rerank_node(state: TaskState, runtime: Runtime[WorkflowContext]):
        query = state.get("query")
        docs = state.get("docs", [])
        
        if not query or not docs:
            logger.debug(f"Rerank节点跳过: query为空={not bool(query)}, docs为空={not bool(docs)}")
            return {"docs": docs}
        
        model_id = RERANK_MODEL_ID
        if model_id is None:
            model_id = getattr(runtime.context, "rerank_model_id", None)
            if model_id is None:
                logger.warning("Rerank模型ID未配置，跳过rerank步骤")
                return {"docs": docs}
        
        try:
            reranker = JinaRerankerComponent(model_id=model_id, top_n=TOP_N)
            reranked_docs = reranker(query=query, docs=docs)
            
            if reranked_docs:
                if hasattr(reranked_docs[0], 'page_content'):
                    docs_list = [doc.page_content for doc in reranked_docs]
                else:
                    docs_list = [str(doc) for doc in reranked_docs]
            else:
                docs_list = []
            
            logger.debug(
                f"Rerank完成: 输入文档数={len(docs)}, "
                f"输出文档数={len(docs_list)}, model_id={model_id}"
            )
            
            return {"docs": docs_list}
        except Exception as e:
            logger.error(f"Rerank错误 {type(e).__name__}: {e}")
            return {"docs": docs}
    
    return rerank_node






from typing import Dict, Any, AsyncGenerator

from langgraph.graph.state import CompiledStateGraph, RunnableConfig

from alayaflow.workflow.runnable.base_runnable_workflow import BaseRunnableWorkflow
from alayaflow.workflow.workflow_info import WorkflowInfo

class StateGraphRunnableWorkflow(BaseRunnableWorkflow):
    def __init__(self, info: WorkflowInfo, graph: CompiledStateGraph) -> None:
        super().__init__(info)
        self._graph = graph

    def invoke(self, inputs: dict, context: dict, config: RunnableConfig) -> dict:
        return self._graph.invoke(inputs, config, context=context)
    
    async def stream_events_async(
        self, 
        inputs: dict, 
        context: dict, 
        config: RunnableConfig
    ) -> AsyncGenerator[Dict[str, Any], None]:
        async for event in self._graph.astream_events(
            inputs, 
            config, 
            version="v2", 
            context=context
        ):
            yield event




from langgraph.graph import StateGraph, START, END
from langgraph.types import Send
from alayaflow.clients.alayamem.http_client import HttpAlayaMemClient

from .schemas import (
    WorkflowInitArgs, 
    OverallState, TaskState, WorkflowContext, Input, Output
)
from typing import Dict, Any
from .nodes.memory_nodes import create_retrieve_node, create_memory_retrieve_node
from .nodes.update_memory_node import create_memory_update_node
from .nodes.llm_extract_node import create_generate_node
from .nodes.validate_node import create_validate_node
from .nodes.plan_node import create_plan_node
from .nodes.rewrite_node import create_rewrite_query_node
from .nodes.rerank_node import create_rerank_node


def create_task_graph(client: HttpAlayaMemClient):
    task_g = StateGraph(TaskState)
    task_g.add_node("rewrite", create_rewrite_query_node())
    task_g.add_node("retrieve", create_retrieve_node(client))
    task_g.add_node("rerank", create_rerank_node())
    task_g.add_node("generate", create_generate_node())

    task_g.add_edge(START, "rewrite")
    task_g.add_edge("rewrite", "retrieve")
    task_g.add_edge("retrieve", "rerank")
    task_g.add_edge("rerank", "generate")
    task_g.add_edge("generate", END)

    return task_g.compile()


def fan_out(state: OverallState):
    memory_json = state.get("memory", "{}")
    return [Send("task", {"task": t, "memory_json": memory_json}) for t in state["tasks"]]


def create_graph(init_args: WorkflowInitArgs | Dict[str, Any]):
    if isinstance(init_args, dict):
        init_args = WorkflowInitArgs(**init_args)
    client = HttpAlayaMemClient(init_args.alayamem_url)

    task_flow = create_task_graph(client)

    g = StateGraph(OverallState, WorkflowContext, input_type=Input, output_type=Output)
    g.add_node("plan", create_plan_node())
    g.add_node("memory", create_memory_retrieve_node(client))
    g.add_node("task", task_flow)
    g.add_node("validate", create_validate_node())
    g.add_node("memory_update", create_memory_update_node(client))

    g.add_edge(START, "plan")
    g.add_edge("plan", "memory")
    g.add_conditional_edges("memory", fan_out, ["task"])
    g.add_edge("task", "validate")
    g.add_edge("validate", "memory_update")
    g.add_edge("memory_update", END)

    return g.compile()



from langgraph.graph import StateGraph, START, END
from langgraph.runtime import Runtime

from alayaflow.component.model import ModelManager

from .schemas import WorkflowInitArgs, WorkflowState, WorkflowContext, Input, Output

def mk_chat_node():
    model_manager = ModelManager()

    def chat_node(state: WorkflowState, runtime: Runtime[WorkflowContext]):
        model_id = runtime.context.chat_model_id
        chat_model = model_manager.get_model(model_id)
        if not chat_model:
            raise ValueError(f"无法找到模型ID为 '{model_id}' 的模型配置")

        messages = state["messages"].copy()
        updated_state = state.copy()

        retrieved_docs = state.get("retrieved_docs", [])
        if retrieved_docs:
            context_text = "\n\n".join([str(doc) for doc in retrieved_docs])
            from langchain_core.messages import SystemMessage
            context_message = SystemMessage(
                content=f"以下是相关的参考资料，请基于这些资料回答用户的问题：\n\n{context_text}"
            )
            messages.insert(0, context_message)

        response = chat_model.invoke(messages)
        updated_state['chat_response'] = response
        return updated_state
    return chat_node


def create_graph(init_args: WorkflowInitArgs | dict):
    if isinstance(init_args, dict):
        init_args = WorkflowInitArgs(**init_args)
    alayamem_url = init_args.alayamem_url

    graph = StateGraph(WorkflowState, WorkflowContext, input_type=Input, output_type=Output)


    graph.add_node("chat_node", mk_chat_node())


    graph.add_edge(START, "chat_node")
    graph.add_edge("chat_node",  END)

    return graph.compile()



import json
from threading import Semaphore

from langgraph.runtime import Runtime

from alayaflow.component.llm_node import LLMComponent, ResponseFormat

from ..schemas import TaskState, WorkflowContext
from ..utils import make_patch


def create_generate_node():
    MAX_CONCURRENCY: int = 10
    limiter = Semaphore(MAX_CONCURRENCY)

    def _build_system_prompt(keys: list[str]) -> str:
        keys_str = ", ".join(keys)

        return f"""
你是一个严谨的“局部字段抽取器”（table patch extractor）。

你的任务是：**只为指定字段抽取值**，严格依据提供的记忆信息和知识片段，不得猜测或编造。

通用规则：
1. 输出必须是严格合法 JSON，不允许包含解释、Markdown、代码块或多余文本。
2. **只允许输出以下字段（不多不少）**：{keys_str}
3. 所有字段值必须是字符串。
4. 找不到 / 不确定 / 空值 / 占位符 → 必须输出空字符串 ""。
5. 字段名可能存在空格或轻微变体（如“姓 名”≈“姓名”），允许智能匹配，但不得扩展到未指定字段。

占位符识别：
- 若候选值是模板占位符或签字日期类文本
（如“签字： 年 月 日”“学院盖章： 年 月 日”等），必须返回 ""。""".strip()

    def _build_user_prompt(memory_json: str, content_text: str, path: list[str], keys: list[str],) -> str:
        path_str = " / ".join(path) if path else "<root>"
        keys_str = ", ".join(keys)

        json_skeleton = "{\n" + ",\n".join([f'  "{k}": ""' for k in keys]) + "\n}"

        return f"""
【本次任务定位】
字段路径（仅用于语义定位，不要输出）：{path_str}
需要抽取的字段：{keys_str}

【用户记忆】
{memory_json}

【知识库片段】
{content_text}

【输出要求】
- 只输出一个 JSON 对象
- key 必须严格为：{keys_str}
- 无法确定 / 空值 / 占位符 → 输出 ""

【JSON 输出模板】
{json_skeleton}""".strip()

    def _non_empty_kv(extracted: dict[str, str]) -> dict[str, str]:
        return {k: v for k, v in extracted.items() if isinstance(v, str) and v.strip()}
    
    def generate_node(state: TaskState, runtime: Runtime[WorkflowContext]):
        task = state["task"]
        path = task.path
        keys = list(task.keys)

        memory_json = state.get("memory_json", "")
        docs = state.get("docs", [])
        task_id = f"{'/'.join(path) or '<root>'}:{','.join(keys)}"
        default_kv = {k: "" for k in keys}
        default_patch = make_patch(path, default_kv)
        if not memory_json and not docs:
            return {
                "final_result": default_patch,
                "context_by_task": {task_id: []},
            }
        formatted_context = "\n\n".join(
            [f"片段 {i+1}:\n{doc}" for i, doc in enumerate(docs)]
        )
        try:
            with limiter:
                system_prompt = _build_system_prompt(keys)
                user_prompt = _build_user_prompt(memory_json, formatted_context, path, keys)

                llm = LLMComponent(
                    model_id=runtime.context.extract_model_id,
                    system_prompt=system_prompt,
                    prompt=user_prompt,
                    response_format=ResponseFormat.JSON,
                    temperature=0.6,
                )

                msg = llm()
                obj = json.loads(msg.content)

                extracted = {}
                for k in keys:
                    v = obj.get(k, "")
                    extracted[k] = (str(v).strip() if v is not None else "")
                patch = make_patch(path, extracted)
                mem_kv = _non_empty_kv(extracted)
                mem_patch = make_patch(path, mem_kv) if mem_kv else {}
                return {
                    "final_result": patch,
                    "memory_patch": mem_patch,
                    "context_by_task": {task_id: docs},
                }

        except Exception as e:
            return {
                "final_result": default_patch,
                "memory_patch": {}, 
                "context_by_task": {task_id: docs},
                "errors": {task_id: f"LLM Error: {type(e).__name__}: {e}"},
            }

    return generate_node



from typing import Generator, Dict, List, Self, Optional, AsyncGenerator

from alayaflow.utils.singleton import SingletonMeta
from alayaflow.workflow import WorkflowManager
from alayaflow.execution import ExecutorManager, ExecutorType
from alayaflow.common.config import settings
from alayaflow.component.model import ModelManager, ModelProfile
from alayaflow.workflow.runnable import BaseRunnableWorkflow
from alayaflow.utils.logger import AlayaFlowLogger


logger = AlayaFlowLogger()


class APISingleton(metaclass=SingletonMeta):
    def __init__(self) -> None:
        self.workflow_manager = WorkflowManager()
        self.executor_manager = ExecutorManager(
            workflow_manager=self.workflow_manager
        )
        self.model_manager = ModelManager()
        self._inited = False                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  

    def is_inited(self) -> bool:
        return self._inited

    def _check_init(self) -> None:
        if not self._inited:
            raise ValueError("Flow APISingleton 未初始化，请先调用 init 方法")

    def init(self, config: dict = None) -> Self:
        """初始化 Flow APISingleton"""
        if config is None:
            config = {}
        # Overwrite all for now
        settings.alayahub_url = config.get("alayahub_url", settings.alayahub_url)
        settings.langfuse_enabled = config.get("langfuse_enabled", settings.langfuse_enabled)
        settings.langfuse_public_key = config.get("langfuse_public_key", settings.langfuse_public_key)
        settings.langfuse_secret_key = config.get("langfuse_secret_key", settings.langfuse_secret_key)
        settings.langfuse_base_url = config.get("langfus_base_url", settings.langfuse_base_url)

        logger.info(f"AlayaFlow is initialized with config: {settings.model_dump()}")
        
        self._inited = True
        return self

    def register_models(self, model_profiles: List[dict | ModelProfile]) -> None:
        self._check_init()
        profiles = [ModelProfile(**profile) if isinstance(profile, dict) else profile for profile in model_profiles]
        for profile in profiles:
            self.model_manager.register_profile(profile)

    def list_loaded_workflow_ids(self) -> List[str]:
        """列举已加载的工作流"""
        self._check_init()
        return self.workflow_manager.list_loaded_workflow_ids()

    def list_available_workflows(self) -> Dict[str, List[str]]:
        """列举工作流，分别返回本地已安装和远程未安装的工作流"""
        self._check_init()
        return self.workflow_manager.list_available_workflows()

    def get_workflow_info(self, workflow_id: str) -> dict:
        """获取工作流信息"""
        self._check_init()
        return self.workflow_manager.get_workflow_info(workflow_id)

    def install_workflow(self, workflow_id: str) -> bool:
        """安装工作流"""
        self._check_init()
        return self.workflow_manager.install_workflow(workflow_id)

    def uninstall_workflow(self, workflow_id: str) -> bool:
        """卸载工作流"""
        self._check_init()
        return self.workflow_manager.uninstall_workflow(workflow_id)

    def load_workflow(self, workflow_id: str, version: str, init_args: Optional[Dict] = None, workflow_dir: Optional[str] = None) -> None:
        """加载本地工作流"""
        self._check_init()
        return self.workflow_manager.load_workflow(workflow_id, version, init_args or {}, workflow_dir)

    def register_workflow(self, runnable: BaseRunnableWorkflow, requirements: List[str] = None) -> None:
        """加载Python项目中的工作流"""
        self._check_init()
        self.workflow_manager.register_workflow(runnable, requirements or [])

    def exec_workflow(
        self,
        workflow_id: str,
        version: str,
        inputs: dict,
        context: dict,
        executor_type: str | ExecutorType = ExecutorType.NAIVE,
        metadata: dict = None
    ) -> Generator[dict, None, None]:
        """执行工作流"""
        self._check_init()
        for event in self.executor_manager.exec_workflow(
            workflow_id=workflow_id,
            version=version,
            inputs=inputs,
            context=context,
            executor_type=executor_type,
            metadata=metadata
        ):
            yield event

    async def exec_workflow_async(
        self,
        workflow_id: str,
        version: str,
        inputs: dict,
        context: dict,
        executor_type: str | ExecutorType = ExecutorType.NAIVE,
        metadata: dict = None
    ) -> AsyncGenerator[dict, None]:
        """异步执行工作流"""
        self._check_init()
        async for event in self.executor_manager.exec_workflow_async(
            workflow_id=workflow_id,
            version=version,
            inputs=inputs,
            context=context,
            executor_type=executor_type,
            metadata=metadata
        ):
            yield event



import asyncio
import traceback
import queue
import threading
from typing import Generator, Dict, Any, AsyncGenerator

from alayaflow.execution.executors.base_executor import BaseExecutor
from alayaflow.utils.coroutine import iter_sync
from alayaflow.workflow.workflow_manager import WorkflowManager
from alayaflow.workflow.runnable import BaseRunnableWorkflow, StateGraphRunnableWorkflow
from alayaflow.common.config import settings
from alayaflow.execution.langfuse_tracing import get_tracing
from alayaflow.utils.logger import AlayaFlowLogger

logger = AlayaFlowLogger()


class NaiveExecutor(BaseExecutor):
    def __init__(self, workflow_manager: WorkflowManager):
        self.workflow_manager = workflow_manager

    def _to_jsonable(self, obj: Any) -> Any:
        """Recursively convert object to JSON-serializable format."""
        if obj is None or isinstance(obj, (bool, int, float, str)):
            return obj
        if isinstance(obj, dict):
            return {k: self._to_jsonable(v) for k, v in obj.items()}
        if isinstance(obj, (list, tuple)):
            return [self._to_jsonable(item) for item in obj]
        if hasattr(obj, 'model_dump') and callable(obj.model_dump):
            return obj.model_dump()
        return str(obj)
    
    def _serialize_event(self, event: Dict) -> Dict:
        # Convert event to JSON-serializable format
        return self._to_jsonable(event)

    def execute_stream(
            self, 
            workflow_id: str, 
            version: str, 
            inputs: dict, 
            context: dict,
            metadata: dict = None
    ) -> Generator[Dict, None, None]:
        return iter_sync(self.execute_stream_async(workflow_id, version, inputs, context, metadata))

    async def execute_stream_async(
            self, 
            workflow_id: str, 
            version: str, 
            inputs: dict, 
            context: dict,
            metadata: dict = None
    ) -> AsyncGenerator[Dict, None]:
        try:
            runnable = self.workflow_manager.get_workflow_runnable(workflow_id, version)
        except ValueError as e:
            yield {"error": str(e), "workflow_id": workflow_id, "version": version}
            return

        logger.info(f"Execute workflow.\n{workflow_id=} {version=}\n{inputs=}\n{context=}")

        # TODO: Support langflow workflow
        # Only support StateGraphRunnableWorkflow for now
        # check it as we pass a langgraph specific config here
        if not isinstance(runnable, StateGraphRunnableWorkflow):
            raise ValueError(f"NaiveExecutor only supports StateGraphRunnableWorkflow, but got {type(runnable)}")

        try:
            tracing = get_tracing(settings)
            langfuse_cb = tracing.build_callback()
        
            if langfuse_cb:
                config = tracing.build_config(inputs, runnable.info, langfuse_cb)
            else:
                config = {}

            if metadata and isinstance(metadata, dict):
                if 'metadata' not in config:
                    config['metadata'] = {}
                config['metadata'].update(metadata)
            
            async for chunk in runnable.stream_events_async(inputs, context, config):
                yield self._serialize_event(chunk)
        except Exception as e:
            yield {"error": str(e), "traceback": traceback.format_exc(), "workflow_id": workflow_id, "version": version}




""" workflow 测试配置文件"""
from pathlib import Path

PROJECT_ROOT = Path(__file__).parent.parent.parent
TEST_WORKFLOW_ID = "simple_chat"
TEST_WORKFLOW_VERSION = "1.0.0"
EXPECTED_WORKFLOW_ID = "simple_chat"
EXPECTED_WORKFLOW_NAME = "Simple Chatbot"
EXPECTED_WORKFLOW_VERSION = "1.0.0"
EXPECTED_ENTRY_POINT = "create_graph"

# 新的目录结构：workflows_dir / workflow_id / version
WORKFLOWS_DIR = PROJECT_ROOT / ".alaya.ai" / "alayaflow" / "workflows"
WORKFLOW_DIR = WORKFLOWS_DIR / TEST_WORKFLOW_ID / TEST_WORKFLOW_VERSION



from typing import Dict, Generator, Any, AsyncGenerator
from enum import Enum

from alayaflow.execution.executors.base_executor import BaseExecutor
from alayaflow.execution.executors.uv_executor import UvExecutor
from alayaflow.execution.executors.naive_executor import NaiveExecutor
from alayaflow.execution.executors.worker_executor import WorkerExecutor
from alayaflow.workflow import WorkflowManager


class ExecutorType(Enum):
    UV = "uv"
    NAIVE = "naive"
    WORKER = "worker"


class ExecutorManager:    
    def __init__(
        self,
        workflow_manager: WorkflowManager
    ):
        self._workflow_manager = workflow_manager
        self._executor_map: Dict[ExecutorType, BaseExecutor] = {}
        self._initialize_executors()
    
    def _initialize_executors(self):
        """Initialize all available executors."""
        self._executor_map[ExecutorType.UV] = UvExecutor(
            workflow_manager=self._workflow_manager
        )
        self._executor_map[ExecutorType.NAIVE] = NaiveExecutor(
            workflow_manager=self._workflow_manager
        )
        self._executor_map[ExecutorType.WORKER] = WorkerExecutor(
            workflow_manager=self._workflow_manager
        )
        
    def init_workflow(self, workflow_id: str, version: str, init_args: dict):
        for executor in self._executor_map.values():
            executor.init_workflow(workflow_id, version, init_args)
    
    def exec_workflow(
            self, 
            workflow_id: str,
            version: str, 
            inputs: dict,
            context: dict, 
            executor_type: ExecutorType | str = ExecutorType.NAIVE,
            metadata: dict = None
    ) -> Generator[Dict[str, Any], None, None]:
        if isinstance(executor_type, str):
            executor_type = ExecutorType(executor_type)
        if executor_type not in self._executor_map:
            raise ValueError(
                f"Unsupported executor kind: {executor_type}. "
                f"Supported kinds: {list(self._executor_map.keys())}"
            )
        executor = self._executor_map[executor_type]
        yield from executor.execute_stream(workflow_id, version, inputs, context, metadata)

    async def exec_workflow_async(
            self, 
            workflow_id: str,
            version: str, 
            inputs: dict,
            context: dict, 
            executor_type: ExecutorType | str = ExecutorType.NAIVE,
            metadata: dict = None
    ) -> AsyncGenerator[Dict[str, Any], None]:
        if isinstance(executor_type, str):
            executor_type = ExecutorType(executor_type)
        if executor_type not in self._executor_map:
            raise ValueError(
                f"Unsupported executor kind: {executor_type}. "
                f"Supported kinds: {list(self._executor_map.keys())}"
            )
        executor = self._executor_map[executor_type]
        async for event in executor.execute_stream_async(workflow_id, version, inputs, context, metadata):
            yield event


from typing import TypedDict, List, Optional

from pydantic import BaseModel, Field

from langchain_core.messages import BaseMessage, AIMessageChunk


class WorkflowInitArgs(BaseModel):
    alayamem_url: str = Field(..., description="AlayaMem URL")


class Input(BaseModel):
    messages: List[BaseMessage] = Field(..., description="List of input messages")


class WorkflowContext(BaseModel):
    user_id: str = Field(..., description="User ID")
    session_id: str = Field(..., description="Session ID")
    chat_model_id: str = Field(..., description="Chat Model ID (AlayaJet is preferred to enable KV cache reuse)")
    save_kv_cache_path: Optional[str] = Field(None, description="Path to save the kvcache file")
    load_kv_cache_path: Optional[str] = Field(None, description="Path to load the kvcache file")


class Output(BaseModel):
    chat_response: dict = Field(..., description="Chat response")


class WorkflowState(TypedDict):
    messages: List[BaseMessage]
    memory_initialized: bool = False
    retrieved_docs: Optional[List[str]]
    stream_chunks: List[AIMessageChunk] = []
    chat_response: Optional[dict]
    context: Optional[str]



from __future__ import annotations

import requests
import urllib.parse
from typing import List, Dict, Any, Optional, Literal

class WebSearchJinaComponent:
    def __init__(
        self,
        *,
        api_key: str,
        
        query: str,
        
        top_k: int = 5,
        site_limit: Optional[str] = None,  
        timeout: int = 60,
        
        no_cache: bool = False,
        
        response_format: Literal['markdown', 'html', 'text'] = 'markdown',
        
        retain_images: Literal['none', 'all', 'occluded'] = 'none',
        
        with_generated_alt: bool = True,
        
        with_links_summary: bool = False,
    ):
        self.api_key = api_key
        self.query = query
        self.top_k = top_k
        self.site_limit = site_limit
        self.timeout = timeout
        
        self.no_cache = no_cache
        self.response_format = response_format
        self.retain_images = retain_images
        self.with_generated_alt = with_generated_alt
        self.with_links_summary = with_links_summary
        
        self.base_url = "https://s.jina.ai/"

    def _construct_query(self) -> str:
        final_query = self.query
        if self.site_limit:
            final_query = f"{final_query} site:{self.site_limit}"
        return final_query

    def _get_headers(self) -> Dict[str, str]:
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Accept": "application/json",
            
            "X-Retain-Images": self.retain_images,
            "X-Return-Format": self.response_format,
            "X-With-Generated-Alt": "true" if self.with_generated_alt else "false",
            "X-With-Links-Summary": "true" if self.with_links_summary else "false",
        }
        
        if self.no_cache:
            headers["X-No-Cache"] = "true"
            
        return headers

    def __call__(self) -> List[Dict[str, Any]]:
        query_str = self._construct_query()
        encoded_query = urllib.parse.quote(query_str)
        url = f"{self.base_url}{encoded_query}"
        
        headers = self._get_headers()

        try:
            response = requests.get(url, headers=headers, timeout=self.timeout)
            
            if response.status_code != 200:
                print(f"Error: API returned status {response.status_code}")
                print(f"Message: {response.text}")
                return []
            json_resp = response.json()
            
            items = []
            if isinstance(json_resp, dict):
                if "data" in json_resp and isinstance(json_resp["data"], list):
                    items = json_resp["data"]
                elif "results" in json_resp:
                    items = json_resp["results"]
            elif isinstance(json_resp, list):
                items = json_resp
            if not items:
                return []
            results = items[:self.top_k]
            clean_results = []
            for item in results:
                entry = {
                    "title": item.get("title", "No Title"),
                    "url": item.get("url", ""),
                    "description": item.get("description", ""),
                    "content": item.get("content", ""), 
                }
                if self.with_links_summary and "links" in item:
                    entry["links"] = item["links"]
                    
                clean_results.append(entry)
                
            return clean_results

        except requests.exceptions.Timeout:
            print("Error: Search request timed out.")
            return []
        except requests.exceptions.RequestException as e:
            print(f"Error calling Jina API: {e}")
            return []
        except Exception as e:
            print(f"Unexpected error: {e}")
            return []

    def to_string_context(self) -> str:
        results = self.__call__()
        if not results:
            return "No search results found."

        parts = []
        for i, item in enumerate(results, 1):
            # limit content length to avoid overly long context
            content_preview = item['content'][:2000] if item['content'] else item['description']
            
            parts.append(
                f"### Result {i}\n"
                f"**Title**: {item['title']}\n"
                f"**Source**: {item['url']}\n"
                f"**Content**:\n{content_preview}\n"
            )
        return "\n\n".join(parts)
    
if __name__ == "__main__":
    import os
    JINA_API_KEY = os.getenv("JINA_API_KEY")
    search_query = "DeepSeek-V3 的技术架构特点"
    search_node = WebSearchJinaComponent(
        api_key=JINA_API_KEY,
        query=search_query,
        top_k=3
    )
    print(f"正在搜索: {search_query} ...")
    search_context = search_node.to_string_context()
    print("搜索完成，部分上下文预览：")
    print(search_context + "...\n")


from pathlib import Path

from alayaflow.workflow import WorkflowInfo

def get_metadata():
    meta = {
        "id": "simple_chat",
        "name": "Simple Chatbot",
        "description": "一个简单的 LLM 对话工作流示例",
        "version": "1.0.0",
        "tags": ["chat", "basic"],
        "entry_file": "workflow.py",
        "entry_point": "create_graph",
        "wf_dir": Path(__file__).parent
    }
    return WorkflowInfo(**meta)



from typing import Optional
from pydantic import BaseModel, Field, ConfigDict, SecretStr


class GenerationConfig(BaseModel):
    model_config = ConfigDict(populate_by_name=True)

    temperature: float = Field(default=0.7, ge=0.0, le=2.0, description="采样温度")
    max_tokens: Optional[int] = Field(default=None, description="最大生成 Token 数")
    top_p: float = Field(default=1.0, description="核采样阈值")
    frequency_penalty: float = Field(default=0.0, ge=-2.0, le=2.0)
    presence_penalty: float = Field(default=0.0, ge=-2.0, le=2.0)
    
    extra_kwargs: dict = Field(default_factory=dict, description="其他参数")


class ModelProfile(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    # Local used fields
    name: str = Field(..., description="模型名称")
    model_id: str = Field(..., description="系统内部使用的唯一 ID")
    model_type: str = Field(default="OpenAI", description="模型类型工厂标识")
    provider_name: str = Field(default="Unknown", description="提供商名称")

    # Connection Credentials
    model_name: str = Field(..., description="厂商的模型名称")
    base_url: str = Field(..., description="API Base URL")
    api_key: SecretStr = Field(default=SecretStr(""), description="API Key")

    # Default Generation Config
    default_config: GenerationConfig = Field(default_factory=GenerationConfig)




from typing import List
from langchain_core.prompts import ChatPromptTemplate
from langchain_core.output_parsers import JsonOutputParser
from langchain_core.exceptions import OutputParserException

from typing import List, Dict, Any, TypedDict, Annotated, Optional
from langchain_core.language_models import BaseChatModel
from pydantic import BaseModel, Field


class IntentionResult(BaseModel):
    """
    意图分类结果模型，标准化返回意图索引和分类原因
    适配 LangChain 结构化输出、JSON 解析器，支持自动类型校验
    """
    # 意图分类索引ID：对应intents列表的下标，兜底时为fallback_index（默认-1）
    classification_id: int = Field(
        ...,
        description="意图分类的索引ID，匹配输入intents列表的下标，兜底值通常为-1",
        gt=-2  # 简单校验：索引ID最小为-1（避免无效负数，可根据业务调整）
    )

    reason: str = Field(
        ...,
        description="意图分类的依据说明，异常场景下为具体错误信息",
        min_length=1  # 简单校验：不可为空字符串
    )

DEFAULT_INTENT_PROMPT_TEMPLATE = """
你是专业的意图分类助手，需严格按照以下规则处理：
1. 从给定的选项中为用户输入匹配唯一的意图，返回对应的索引ID（fallback_index={fallback_index} 为兜底选项，仅当无匹配时使用）
2. 必须以 **纯JSON格式** 输出，字段为 classification_id（int类型）和 reason（str类型），无其他多余文本、注释
3. reason 需简洁说明分类依据，不超过50字

可用意图选项：
{intents}
"""

def execute_intent_classification(
    query: str, 
    intents: List[str], 
    model: BaseChatModel,
    fallback_index: int = -1
) -> IntentionResult:
    """
    核心业务逻辑：执行意图分类（适配所有支持JSON输出的大模型）。
    """
    if not intents:
        return IntentionResult(classification_id=fallback_index, reason='input intent list is empty')

    parser = JsonOutputParser(pydantic_object=IntentionResult)

    formatted_intents = "\n".join([f"选项{i}: {intent}" for i, intent in enumerate(intents)])
    prompt = ChatPromptTemplate.from_template(DEFAULT_INTENT_PROMPT_TEMPLATE)
    full_prompt = prompt.format_prompt(
        fallback_index=fallback_index,
        intents=formatted_intents
    )

    user_prompt = f"### 用户输入\n{query}\n\n{parser.get_format_instructions()}"

    try:
        # 显式指定 response_format={"type": "json_object"}，强制模型返回JSON
        response = model.invoke(
            [
                ("system", full_prompt.to_string()),
                ("user", user_prompt)
            ],
            response_format={"type": "json_object"}
        )
        result_dict = parser.parse(response.content)
        return IntentionResult(**result_dict)
    except OutputParserException as e:
        error_msg = f"JSON解析失败，模型输出非标准格式：{str(e)}"
        import sys
        sys.stderr.write(f"Error during classification: {error_msg}\n")
        return IntentionResult(classification_id=fallback_index, reason=error_msg)
    except Exception as e:
        error_msg = f"分类过程中发生错误：{str(e)}"
        import sys
        sys.stderr.write(f"Error during classification: {error_msg}\n")
        return IntentionResult(classification_id=fallback_index, reason=error_msg)


# def inner_test():
#     from alayaflow.component.chat_model import mk_chat_model_deepseek
#     model = mk_chat_model_deepseek()
#     query = "你好"
#     intents = ["问候", "查询"]
#     result = execute_intent_classification(query, intents, model)
#     print(result)

# if __name__ == "__main__":
#     inner_test()


from alayaflow.workflow.workflow_info import WorkflowInfo
from alayaflow.workflow.workflow_loader import WorkflowLoader
from alayaflow.workflow.workflow_manager import WorkflowManager

__all__ = [
    "WorkflowInfo", 
    "WorkflowLoader", 
    "WorkflowManager",
]




from dataclasses import dataclass
from pathlib import Path
from typing import List, Callable, Any, NamedTuple


class WorkflowKey(NamedTuple):
    id: str
    version: str


@dataclass
class WorkflowInfo:
    """工作流元信息"""
    id: str
    name: str
    description: str
    version: str
    tags: List[str]
    entry_file: str
    entry_point: str
    wf_dir: Path

    @property
    def key(self) -> WorkflowKey:
        return WorkflowKey(self.id, self.version)

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "name": self.name,
            "description": self.description,
            "version": self.version,
            "tags": self.tags,
            "entry_file": self.entry_file,
            "entry_point": self.entry_point,
            "wf_dir": str(self.wf_dir),
        }

    @classmethod
    def from_dict(cls, data: dict) -> "WorkflowInfo":
        return cls(
            id=data["id"],
            name=data["name"],
            description=data["description"],
            version=data["version"],
            tags=data["tags"],
            entry_file=data["entry_file"],
            entry_point=data["entry_point"],
            wf_dir=Path(data.get("wf_dir", "")),
        )



"""AlayaMem 客户端类型定义"""
from typing import List, Any, Dict, TypedDict, Optional


class HybridQueryResults(TypedDict):
    """混合检索结果数据结构"""
    ids: List[List[str]]  
    documents: List[List[str]]
    metadatas: List[List[Dict[str, Any]]]
    distances: List[List[float]]


class HybridQueryResponse(TypedDict):
    """混合检索API返回格式"""
    msg: str
    results: HybridQueryResults
    query: str
    user_id: str
    count: int


class ProfileData(TypedDict, total=False):
    """用户画像数据"""
    profile_id: str
    user_id: str
    name: str
    type: Optional[str]
    created_at: float
    updated_at: float
    content: Dict[str, Any]


class GetProfileResponse(TypedDict):
    """获取用户画像API返回格式"""
    code: int
    msg: str
    data: ProfileData


class UpdateProfileResponse(TypedDict):
    """更新用户画像API返回格式"""
    code: int
    msg: str
    data: ProfileData




from pathlib import Path

from alayaflow.workflow import WorkflowInfo

def get_metadata():
    meta = {
        "id": "autotable",
        "name": "Auto Table Filler",
        "description": "基于知识库的智能表格自动填写工作流",
        "version": "1.0.0",
        "tags": ["extraction", "table", "rag"],
        "entry_file": "workflow.py",
        "entry_point": "create_graph",
        "wf_dir": Path(__file__).parent
    }
    return WorkflowInfo(**meta)




from typing import List, Dict, Tuple, Any, Optional
from collections import defaultdict
from ..schemas import OverallState, GroupTask, FieldSpec


def flatten_leaf_tasks(specs: List[FieldSpec], base_path: Optional[List[str]] = None) -> List[Tuple[Tuple[str, ...], str]]:
    """
    返回：[(path_tuple, leaf_key), ...]
    """
    base_path = base_path or []
    out: List[Tuple[Tuple[str, ...], str]] = []

    def _as_list(x: Any) -> List[Any]:
        if x is None:
            return []
        if isinstance(x, list):
            return x
        return [x]

    for item in specs or []:
        if isinstance(item, str):
            out.append((tuple(base_path), item))
            continue

        if isinstance(item, dict):
            for parent, children in item.items():
                for child in _as_list(children):
                    if isinstance(child, str):
                        out.append((tuple(base_path + [parent]), child))
                    elif isinstance(child, dict):
                        out.extend(flatten_leaf_tasks([child], base_path + [parent]))
                    else:
                        pass
            continue

    return out


def create_plan_node():
    """创建规划节点：将输入的字段模板规划为多个独立的抽取任务"""
    def plan_node(state: OverallState):
        leaf = flatten_leaf_tasks(state["fields"])
        grouped: Dict[Tuple[str, ...], List[str]] = defaultdict(list)
        for path, key in leaf:
            grouped[path].append(key)

        tasks: List[GroupTask] = []
        for path, keys in grouped.items():
            # 去重保持顺序
            seen = set()
            uniq = []
            for k in keys:
                if k not in seen:
                    seen.add(k)
                    uniq.append(k)
            tasks.append(GroupTask(path=path, keys=tuple(uniq)))

        tasks.sort(key=lambda t: (len(t.path), t.path))
        return {"tasks": tasks}
    
    return plan_node




import sys
import json
import importlib.util
import socket
import struct
import traceback
import asyncio
from pathlib import Path

# 尝试导入 langchain 的序列化工具 (如果环境中有)
try:
    from langchain_core.messages import BaseMessage
    from pydantic import BaseModel
except ImportError as e:
    raise ImportError("请安装 langchain_core 和 pydantic 库以支持 LangChain/LangGraph 序列化") from e

try:
    import alayaflow
except ImportError as e:
    raise ImportError("alayaflow没有在worker内安装") from e

try:
    from alayaflow.workflow.workflow_loader import WorkflowLoader
    from alayaflow.common.config import settings
    from alayaflow.execution.langfuse_tracing import get_tracing
except ImportError as e:
    raise e

port = int(sys.argv[1])
wf_dir = sys.argv[2]

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
try:
    s.connect(('127.0.0.1', port))
except ConnectionRefusedError:
    sys.exit(1)

def default_encoder(obj):
    if hasattr(obj, 'model_dump') and callable(obj.model_dump):
        return obj.model_dump()
    return str(obj)

def send_chunk(data_dict):
    try:
        json_bytes = json.dumps(data_dict, default=default_encoder).encode('utf-8')
        header = struct.pack('!I', len(json_bytes))
        s.sendall(header + json_bytes)
    except Exception as e:
        err_msg = json.dumps({"error": f"Serialization Error: {str(e)}"}).encode('utf-8')
        s.sendall(struct.pack('!I', len(err_msg)) + err_msg)

async def run():
    try:
        input_str = sys.stdin.read()
        lines = input_str.strip().split('\n')
        init_args = json.loads(lines[0]) if len(lines) > 0 and lines[0].strip() else {}
        config = json.loads(lines[1]) if len(lines) > 1 and lines[1].strip() else {}
        input_data = json.loads(lines[2]) if len(lines) > 2 and lines[2].strip() else {}

        print("Input data for workflow:", input_data)

        # 加载工作流元信息和创建器
        # wf_dir 格式: {workflows_dir}/{workflow_id}/{version}
        wf_dir_path = Path(wf_dir)
        workflows_dir = str(wf_dir_path.parent.parent)  # 工作流根目录
        workflow_id = wf_dir_path.parent.name  # 工作流ID
        version = wf_dir_path.name  # 版本号

        workflow_dir = Path(workflows_dir) / workflow_id / version
        
        loader = WorkflowLoader(workflow_dir, workflow_id, version)
        result = loader.load_workflow()
        wf_info = result.info
        creator = result.creator
        graph = creator(init_args)

        tracing = get_tracing(settings)
        langfuse_cb = tracing.build_callback()
        
        # 合并用户提供的 config 和 tracing config
        merged_config = dict(config) if config else {}
        if langfuse_cb:
            merged_config.update(tracing.build_config(input_data, wf_info, langfuse_cb))
        
        async for chunk in graph.astream_events(input_data, config=merged_config, version="v2"):
            send_chunk(chunk)

    except Exception as e:
        error_info = {
            "error": str(e),
            "traceback": traceback.format_exc()
        }
        send_chunk(error_info)
        traceback.print_exc()
        print(f"[Runner Critical] {{error_info}}", file=sys.stderr)

    finally:
        s.close()

asyncio.run(run())



from abc import ABC, abstractmethod
from typing import Dict, List, Any

from alayaflow.clients.alayamem.types import (
    HybridQueryResponse,
    GetProfileResponse,
    UpdateProfileResponse,
)

class BaseAlayaMemClient(ABC):


    @abstractmethod
    def query_file(self, user_id: str, message: Any, limit: int = 10, use_query_params: bool = False) -> Dict:
        """ 查询文件内容 - POST 请求 """
        pass

    @abstractmethod
    def commit_turn(self, session_id: str, user_text: str, assistant_text: str,
                    user_id: str = None, window_size: int = 10) -> Dict:
        """ 提交多轮对话记录 - POST 请求 """
        pass

    @abstractmethod
    def turns(self, session_id: str, user_id: str = None, limit: int = 10) -> List:
        """ 查询对话历史记录 - GET 请求 """
        pass

    @abstractmethod
    def summary(self, session_id: str, user_id: str = None) -> Dict:
        """ 查询会话摘要 - GET 请求 """
        pass

    @abstractmethod
    def profile(self, user_id: str) -> Dict:
        """ 查询用户画像 - GET 请求 """
        pass

    @abstractmethod
    def hybrid_query(self, query: str, user_id: str, limit: int = 10) -> HybridQueryResponse:
        """
        混合检索 - POST 请求
        
        Returns:
            HybridQueryResponse: 包含检索结果的字典，结构见 types.HybridQueryResponse
        """
        pass

    @abstractmethod
    def get_profile(self, user_id: str, profile_id: str) -> GetProfileResponse:
        """
        获取用户画像 - GET 请求
        
        Returns:
            GetProfileResponse: 包含画像数据的字典，结构见 types.GetProfileResponse
        """
        pass

    @abstractmethod
    def update_profile(self, user_id: str, profile_id: str, content: Dict[str, Any]) -> UpdateProfileResponse:
        """
        更新用户画像 - PUT 请求
        
        Returns:
            UpdateProfileResponse: 包含更新结果的字典，结构见 types.UpdateProfileResponse
        """
        pass


import json
from typing import Dict, Any, Tuple, List, Optional
from alayaflow.clients.alayamem.http_client import HttpAlayaMemClient
from alayaflow.clients.alayamem.types import HybridQueryResponse, HybridQueryResults
from alayaflow.utils.logger import AlayaFlowLogger


logger = AlayaFlowLogger()

class RetrieveComponent:
    def __init__(self, client: HttpAlayaMemClient):
        self.client = client
    
    def __call__(self, query: str, limit: int = 5, user_id: str = None) -> List[str]:
        try:
            if not user_id:
                logger.info(f"RetrieveComponent: user_id is None, query: {query}")
                return []
            result = self.client.query_file(user_id=user_id, message=query, limit=limit)
            # query_file 返回格式: {'msg': 'query_file', 'results': {'documents': [['doc1', 'doc2', ...]], ...}, ...}
            if isinstance(result, dict):
                results = result.get('results', {})
                if isinstance(results, dict):
                    documents = results.get('documents', [])
                    # documents 格式是 [['doc1', 'doc2', ...]]，需要取第一个元素
                    if documents and isinstance(documents, list) and len(documents) > 0:
                        if isinstance(documents[0], list):
                            return documents[0]
                        return documents
            return []
        except Exception as e:
            logger.error(f"RetrieveComponent: exception: {type(e).__name__}: {e}")
            return []


class HybridRetrieveComponent:
    """混合检索组件，结合向量检索和关键词检索，返回检索结果"""
    def __init__(self, client: HttpAlayaMemClient):
        self.client = client
    
    def __call__(self, query: str, limit: int = 5, user_id: str = None) -> Optional[HybridQueryResults]:
        """
        执行混合检索，返回检索结果
        
        Args:
            query: 查询文本
            limit: 返回结果数量，默认 5
            user_id: 用户ID
            
        Returns:
            HybridQueryResults: 包含以下字段：
                - ids: 文档ID列表 (List[List[str]])
                - documents: 文档内容列表 (List[List[str]])
                - metadatas: 元数据列表 (List[List[Dict[str, Any]]])
                - distances: 距离列表 (List[List[float]])
        """
        try:
            if not user_id:
                logger.info(f"HybridRetrieveComponent: user_id is None, query: {query}")
                return None
            response: HybridQueryResponse = self.client.hybrid_query(query=query, user_id=user_id, limit=limit)
            return response.get('results')
        except Exception as e:
            logger.error(f"HybridRetrieveComponent: exception: {type(e).__name__}: {e}")
            return None


class MemoryRetrieveComponent:
    def __init__(self, client: HttpAlayaMemClient):
        self.client = client

    def __call__(self, user_id: str, profile_id: str) -> str:
        try:
            memory_obj = self.client.get_profile(user_id=user_id, profile_id=profile_id)
            # 提取 content 部分：返回格式 {'code': 0, 'msg': 'success', 'data': {'content': {...}, ...}}
            if isinstance(memory_obj, dict):
                data = memory_obj.get('data', {})
                if isinstance(data, dict):
                    content = data.get('content', {})
                    return json.dumps(content, ensure_ascii=False)
            return json.dumps(memory_obj, ensure_ascii=False)
        except Exception:
            return ""


class MemoryUpdateComponent:
    def __init__(self, client: HttpAlayaMemClient):
        self.client = client

    def __call__(self, user_id: str, patch: Dict[str, Any], profile_id: str) -> Tuple[str, bool]:
        try:
            resp = self.client.update_profile(user_id=user_id, profile_id=profile_id, content=patch)
            return json.dumps(resp or {}, ensure_ascii=False), True
        except Exception:
            return "", False
